package sun.reflect.generics.factory;

import java.lang.reflect.Array;
import sun.reflect.generics.reflectiveObjects.GenericArrayTypeImpl;
import sun.reflect.generics.reflectiveObjects.ParameterizedTypeImpl;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import sun.reflect.generics.reflectiveObjects.WildcardTypeImpl;
import java.lang.reflect.WildcardType;
import sun.reflect.generics.reflectiveObjects.TypeVariableImpl;
import java.lang.reflect.TypeVariable;
import sun.reflect.generics.tree.FieldTypeSignature;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import sun.reflect.generics.scope.Scope;
import java.lang.reflect.GenericDeclaration;

public class CoreReflectionFactory implements GenericsFactory
{
    private final GenericDeclaration decl;
    private final Scope scope;
    
    private CoreReflectionFactory(GenericDeclaration decl, Scope scope) {
        this.decl = decl;
        this.scope = scope;
    }
    
    private GenericDeclaration getDecl() {
        return this.decl;
    }
    
    private Scope getScope() {
        return this.scope;
    }
    
    private ClassLoader getDeclsLoader() {
        if ((this.decl instanceof Class)) {
            return ((Class)this.decl).getClassLoader();
        }
        if ((this.decl instanceof Method)) {
            return ((Method)this.decl).getDeclaringClass().getClassLoader();
        }
        assert !(!(this.decl instanceof Constructor)) : "Constructor expected";
        return ((Constructor)this.decl).getDeclaringClass().getClassLoader();
    }
    
    public static CoreReflectionFactory make(GenericDeclaration genericDeclaration, Scope scope) {
        return new CoreReflectionFactory(genericDeclaration, scope);
    }
    
    @Override
    public TypeVariable<?> makeTypeVariable(String name, FieldTypeSignature[] bs) {
        return TypeVariableImpl.make(this.getDecl(), name, bs, this);
    }
    
    @Override
    public WildcardType makeWildcard(FieldTypeSignature[] ubs, FieldTypeSignature[] lbs) {
        return WildcardTypeImpl.make(ubs, lbs, this);
    }
    
    @Override
    public ParameterizedType makeParameterizedType(Type type, Type[] actualTypeArguments, Type ownerType) {
        return ParameterizedTypeImpl.make((Class)type, actualTypeArguments, ownerType);
    }
    
    @Override
    public TypeVariable<?> findTypeVariable(String s) {
        return this.getScope().lookup(s);
    }
    
    @Override
    public Type makeNamedType(String s) {
        try {
            return Class.forName(s, false, this.getDeclsLoader());
        }
        catch (ClassNotFoundException cause) {
            throw new TypeNotPresentException(s, cause);
        }
    }
    
    @Override
    public Type makeArrayType(Type ct) {
        if ((ct instanceof Class)) {
            return Array.newInstance((Class)ct, 0).getClass();
        }
        return GenericArrayTypeImpl.make(ct);
    }
    
    @Override
    public Type makeByte() {
        return Byte.TYPE;
    }
    
    @Override
    public Type makeBool() {
        return Boolean.TYPE;
    }
    
    @Override
    public Type makeShort() {
        return Short.TYPE;
    }
    
    @Override
    public Type makeChar() {
        return Character.TYPE;
    }
    
    @Override
    public Type makeInt() {
        return Integer.TYPE;
    }
    
    @Override
    public Type makeLong() {
        return Long.TYPE;
    }
    
    @Override
    public Type makeFloat() {
        return Float.TYPE;
    }
    
    @Override
    public Type makeDouble() {
        return Double.TYPE;
    }
    
    @Override
    public Type makeVoid() {
        return Void.TYPE;
    }
}
